package contextualbreakpoint.ui.utils;

import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.jdt.core.IMember;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.Signature;
import org.eclipse.jdt.internal.debug.ui.BreakpointUtils;

import contextualbreakpoints.breakpoints.IJavaContextualBreakpoint;
import contextualbreakpoints.breakpoints.IJavaContextualLineBreakpoint;
import contextualbreakpoints.breakpoints.IJavaContextualMethodBreakpoint;
import contextualbreakpoints.contextlogic.MethodKey;
import contextualbreakpoints.pointcuts.AfterPointcut;
import contextualbreakpoints.pointcuts.AndPointcut;
import contextualbreakpoints.pointcuts.BeforePointcut;
import contextualbreakpoints.pointcuts.CFlowPointcut;
import contextualbreakpoints.pointcuts.LineExecutionPointcut;
import contextualbreakpoints.pointcuts.MethodExecutionPointcut;
import contextualbreakpoints.pointcuts.NotPointcut;
import contextualbreakpoints.pointcuts.NullPointcut;
import contextualbreakpoints.pointcuts.OrPointcut;
import contextualbreakpoints.pointcuts.Pointcut;

/**
 * Utility class to compose friendly names for breakpoints, method keys, pointcuts
 * @author Rick
 *
 */
public class FriendlyNames {
	
	public static void testmeth() {
		
	}
	
	public static String getFriendlyName(Object o) {
		if (o instanceof IJavaContextualBreakpoint) {
			return getFriendlyBreakpointName((IJavaContextualBreakpoint)o);
		}
		if (o instanceof MethodKey) {
			return getFriendlyMethodKeyName((MethodKey)o);
		}
		if (o instanceof Pointcut) {
			return getFriendlyPointcutName((Pointcut)o);
		}
		return ""; //$NON-NLS-1$
	}
	
	private static String getLeftPointcutBracket() {
		return Messages.getString("FriendlyNames.leftbracket");  //$NON-NLS-1$
	}
	
	private static String getRightPointcutBracket() {
		return Messages.getString("FriendlyNames.rightbracket");  //$NON-NLS-1$
	}
	
	private static String getFriendlyPointcutName(Pointcut pointcut) {
		if (pointcut instanceof AfterPointcut) {
			Pointcut child = ((AfterPointcut)pointcut).getPointcut();
			return Messages.getString("FriendlyNames.0") + getLeftPointcutBracket() + getFriendlyPointcutName(child) + getRightPointcutBracket();  //$NON-NLS-1$
		}
		if (pointcut instanceof AndPointcut) {
			String name = ""; //$NON-NLS-1$
			List pointcuts = ((AndPointcut)pointcut).getPointcuts();
			for (Iterator i = pointcuts.iterator(); i.hasNext();) {
				Pointcut pc = (Pointcut)i.next();
				// name = name + getFriendlyPointcutName(pc);
				if (pc instanceof AndPointcut || pc instanceof OrPointcut) {
					name = name + getLeftPointcutBracket() + getFriendlyPointcutName(pc) + getRightPointcutBracket();
				} else {
					name = name + getFriendlyPointcutName(pc);
				}
				
				if (i.hasNext()) {
					//name = name + ", "; //$NON-NLS-1$
					name = name + " &&&& ";  //$NON-NLS-1$
				}
			}
			return name;
			//return Messages.getString("FriendlyNames.1") + getLeftPointcutBracket() + name + getRightPointcutBracket(); //$NON-NLS-1$
		}
		if (pointcut instanceof BeforePointcut) {
			Pointcut child = ((BeforePointcut)pointcut).getPointcut();
			return Messages.getString("FriendlyNames.2") + getLeftPointcutBracket() + getFriendlyPointcutName(child) + getRightPointcutBracket();  //$NON-NLS-1$
		}
		if (pointcut instanceof CFlowPointcut) {
			Pointcut child = ((CFlowPointcut)pointcut).getPointcut();
			return Messages.getString("FriendlyNames.3") + getLeftPointcutBracket() + getFriendlyPointcutName(child) + getRightPointcutBracket();  //$NON-NLS-1$
		}
		if (pointcut instanceof LineExecutionPointcut) {
			IJavaContextualLineBreakpoint bp = ((LineExecutionPointcut)pointcut).getOriginBreakpoint();
			return Messages.getString("FriendlyNames.4") + getLeftPointcutBracket() + getFriendlyLineName(bp) + getRightPointcutBracket();  //$NON-NLS-1$
		}
		if (pointcut instanceof MethodExecutionPointcut) {
			MethodKey methKey = ((MethodExecutionPointcut)pointcut).getMethodKey();
			return Messages.getString("FriendlyNames.5") + getLeftPointcutBracket() + getFriendlyMethodKeyName(methKey) + getRightPointcutBracket();  //$NON-NLS-1$
		}
		if (pointcut instanceof NotPointcut) {
			Pointcut child = ((NotPointcut)pointcut).getPointcut();
			if (child instanceof AndPointcut || child instanceof OrPointcut) {
				return Messages.getString("FriendlyNames.6") + getLeftPointcutBracket() + getFriendlyPointcutName(child) + getRightPointcutBracket();  //$NON-NLS-1$
			}
			return Messages.getString("FriendlyNames.6") + getFriendlyPointcutName(child); //$NON-NLS-1$
		}
		if (pointcut instanceof NullPointcut) {
			return Messages.getString("FriendlyNames.7");  //$NON-NLS-1$
		}
		if (pointcut instanceof OrPointcut) {
			String name = ""; //$NON-NLS-1$
			List pointcuts = ((OrPointcut)pointcut).getPointcuts();
			for (Iterator i = pointcuts.iterator(); i.hasNext();) {
				Pointcut pc = (Pointcut)i.next();
				// name = name + getFriendlyPointcutName(pc);
				if (pc instanceof AndPointcut || pc instanceof OrPointcut) {
					name = name + getLeftPointcutBracket() + getFriendlyPointcutName(pc) + getRightPointcutBracket();
				} else {
					name = name + getFriendlyPointcutName(pc);
				}
				if (i.hasNext()) {
					// name = name + ", "; //$NON-NLS-1$
					name = name + " || "; //$NON-NLS-1$
				}
			}
			return name;
			//return Messages.getString("FriendlyNames.8") + getLeftPointcutBracket() + name + getRightPointcutBracket();  //$NON-NLS-1$
		}
		System.out.println("Unrecognized pointcut: " + pointcut);
		return "";  //$NON-NLS-1$
	}

	private static String getFriendlyMethodKeyName(MethodKey key) {
		String name = ""; //$NON-NLS-1$
		String sig = getFriendlyMethodSignature(key.getMethodName(), key.getMethodSignature());
		name = key.getClassName() + "." + sig; //$NON-NLS-1$
		return name;
	}
	
	private static String getFriendlyMethodSignature(String methName, String rawSig) {
		return Signature.toString(rawSig, methName, null, true, false);
	}
	
	private static String getFriendlyLineName(IJavaContextualLineBreakpoint bp) {
		try {
			int lineNum = bp.getLineNumber();
			String typeName = bp.getTypeName();
			IMember member = BreakpointUtils.getMember(bp);
			String sig = ""; //$NON-NLS-1$
			if (member instanceof IMethod) {
				IMethod method = (IMethod)member;
				sig = getFriendlyMethodSignature(method.getElementName(), method.getSignature());
			} else {
				System.err.println("member not instnaceof method???");
			}
			String finalString = typeName + " [line " + lineNum + "] - " + sig; //$NON-NLS-1$ //$NON-NLS-2$
			return finalString;
		} catch (CoreException e) {
			System.err.println("Something wrong with getting friendly line name");
			e.printStackTrace();
			return ""; //$NON-NLS-1$
		}
	}

	private static String getFriendlyBreakpointName(IJavaContextualBreakpoint bp) {
		if (bp instanceof IJavaContextualLineBreakpoint) {
			IJavaContextualLineBreakpoint linebp = (IJavaContextualLineBreakpoint)bp;
			return getFriendlyLineName(linebp);
		}
		if (bp instanceof IJavaContextualMethodBreakpoint) {
			IJavaContextualMethodBreakpoint methbp = (IJavaContextualMethodBreakpoint)bp;
			return getFriendlyMethodBreakpointName(methbp);
		}
		return ""; //$NON-NLS-1$
	}

	private static String getFriendlyMethodBreakpointName(IJavaContextualMethodBreakpoint methbp) {
		try {
			String typeName = methbp.getTypeName();
			String sig = getFriendlyMethodSignature(methbp.getMethodName(), methbp.getMethodSignature());
			String entryexit = ""; //$NON-NLS-1$
			if (methbp.isEntry()) {
				entryexit = "entry"; //$NON-NLS-1$
				if (methbp.isExit()) {
					entryexit += " and exit"; //$NON-NLS-1$
				}
			} 
			if (methbp.isExit()) {
				entryexit = "exit"; //$NON-NLS-1$
			}
			String finalString = typeName + " [" + entryexit + "] - " + sig;  //$NON-NLS-1$//$NON-NLS-2$
			return finalString;
		} catch (CoreException e) {
			System.out.println("something wrong with getting friendly method name");
			e.printStackTrace();
			return ""; //$NON-NLS-1$
		}
	}
	
}
