package contextualbreakpoints.ui.actions;

import org.eclipse.jdt.internal.debug.ui.actions.AbstractBreakpointRulerAction;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuCreator;
import org.eclipse.jface.text.source.IVerticalRulerInfo;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MenuAdapter;
import org.eclipse.swt.events.MenuEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.ui.IEditorActionDelegate;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowPulldownDelegate2;
import org.eclipse.ui.texteditor.ITextEditor;

public abstract class BreakpointConditionContextMenuAction extends AbstractBreakpointRulerAction implements IEditorActionDelegate, IMenuCreator, IWorkbenchWindowPulldownDelegate2{
	
	protected ITextEditor editor;
	protected IVerticalRulerInfo rulerInfo;
	
	public BreakpointConditionContextMenuAction(ITextEditor editor, IVerticalRulerInfo rulerInfo) {
		this.editor = editor;
		this.rulerInfo = rulerInfo;
		
	}
	
	public Menu getMenu(Control parent) {
		// never called
		return null;
	}
	
	public void dispose() {
	}

	public void init(IWorkbenchWindow window) {
	}

	public void run(IAction action) {
		// we become only an IMenuCreator
	}
	
	public void selectionChanged(IAction action, ISelection selection) {
		action.setMenuCreator(this);
		action.setEnabled(true);
	}

	public Menu getMenu(Menu parent) {
		Menu menu = new Menu(parent);
		
		/**
		 * Add listener to repopulate the menu each time
		 * it is shown because MenuManager.update(boolean, boolean) 
		 * doesn't dispose pulldown ActionContribution items for each popup menu.
		 */
		menu.addMenuListener(new MenuAdapter() {
			public void menuShown(MenuEvent e) {
				Menu m = (Menu)e.widget;
				MenuItem[] items = m.getItems();
				for (int i=0; i < items.length; i++) {
					items[i].dispose();
				}
				fillMenu(m);
			}
		});
		fillMenu(menu);
		return menu;
	}
	
	protected abstract void fillMenu(Menu menu);

	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
		// not called
	}

	public void setActiveEditor(IAction action, IEditorPart targetEditor) {
		action.setMenuCreator(this);
	}
	
	protected void addActionToMenu(Menu menu, BreakpointConditionBreakpointRulerAction action) {
		class MySelectionListener extends SelectionAdapter {
			
			private IAction action; // action to take
			
			public MySelectionListener(IAction action) {	
				super();
				this.action = action;
			}
			
			public void widgetSelected(SelectionEvent e) {
				action.run();
			}
		}
		
		class MyMenuListener extends MenuAdapter {
			
			private BreakpointConditionBreakpointRulerAction action; // action to take
			
			public MyMenuListener(BreakpointConditionBreakpointRulerAction action) {	
				super();
				this.action = action;
			}
			
			public void menuShown(MenuEvent e) {
				action.update();
			}
		}
		
		MenuItem item = new MenuItem(menu, SWT.CHECK);
		item.setText(action.getText());
		item.addSelectionListener(new MySelectionListener(action));
		
		menu.addMenuListener(new MyMenuListener(action));
	
	}
	
	public void update() {
	}

}
