package contextualbreakpoints.ui.listeners;

import java.util.List;

import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;

import contextualbreakpoints.breakpoints.IJavaContextualBreakpoint;
import contextualbreakpoints.breakpoints.IJavaContextualLineBreakpoint;
import contextualbreakpoints.contextlogic.MethodKey;
import contextualbreakpoints.pointcuts.AfterPointcut;
import contextualbreakpoints.pointcuts.AndPointcut;
import contextualbreakpoints.pointcuts.BeforePointcut;
import contextualbreakpoints.pointcuts.CFlowPointcut;
import contextualbreakpoints.pointcuts.LineExecutionPointcut;
import contextualbreakpoints.pointcuts.NotPointcut;
import contextualbreakpoints.pointcuts.OrPointcut;
import contextualbreakpoints.pointcuts.Pointcut;
import contextualbreakpoints.ui.widgets.ContextDialog;
import contextualbreakpoints.pointcuts.MethodExecutionPointcut;

public class ContextApplyListener implements SelectionListener {
	
	ContextDialog dialog;
	IJavaContextualBreakpoint breakpoint; // breakpoint to pass to dialog;
	
	public void widgetSelected(SelectionEvent e) {
		// add to the breakpoint's context
		if (dialog != null && breakpoint != null) {
//			condition should be of the structure 
//                       AndCondition
//                         |       |
//			       NotCondition   AndCondition
//			         |    
//			OrCondition   
//			
//			It could be possible that the condition is not of this form
//			in which case we don't know what to do
			Pointcut cond = breakpoint.getContextCondition();
			
			if (cond instanceof AndPointcut) {
				AndPointcut andCond = (AndPointcut)cond;
				List conds = andCond.getPointcuts();
				if (conds.size() == 2) {
					if (conds.get(0) instanceof NotPointcut) {
						NotPointcut notCond = (NotPointcut)conds.get(0);
						if (notCond.getPointcut() instanceof OrPointcut) {
							OrPointcut exceptCond = (OrPointcut)notCond.getPointcut();
							if (conds.get(1) instanceof AndPointcut) {
								AndPointcut onlyCond = (AndPointcut)conds.get(1);
								applyExceptGroup(exceptCond);
								applyOnlyGroup(onlyCond);
							}
						}
					}
				}
			}
			if (cond instanceof NotPointcut) {
				
			}
			dialog.updateBreakpointContextExpression(breakpoint);
		}
	}
	
	private void applyExceptGroup(OrPointcut exceptCond) {
		// add the selected conditions to the exceptions
		Table beforeTable = dialog.getExceptBeforeTable();
		Table afterTable = dialog.getExceptAfterTable();
		Table cflowTable = dialog.getExceptCFlowTable();
		
		TableItem[] beforeItems = beforeTable.getItems();
		for (int i = 0; i < beforeItems.length; i++) {
			if (beforeItems[i].getChecked()) {
				if (beforeItems[i].getData() instanceof IJavaContextualLineBreakpoint) {
					IJavaContextualLineBreakpoint bp = (IJavaContextualLineBreakpoint)beforeItems[i].getData();
					BeforePointcut beforeCond = new BeforePointcut();
					LineExecutionPointcut lep = new LineExecutionPointcut(bp);
					beforeCond.setPointcut(lep);
					exceptCond.addPointcut(beforeCond);
				}
			}
		}
		
		TableItem[] afterItems = afterTable.getItems();
		for (int i = 0; i < afterItems.length; i++) {
			if (afterItems[i].getChecked()) {
				if (afterItems[i].getData() instanceof IJavaContextualLineBreakpoint) {
					IJavaContextualLineBreakpoint bp = (IJavaContextualLineBreakpoint)afterItems[i].getData();
					AfterPointcut afterCond = new AfterPointcut();
					LineExecutionPointcut lep = new LineExecutionPointcut(bp);
					afterCond.setPointcut(lep);
					exceptCond.addPointcut(afterCond);
				}
			}
		}
		
		TableItem[] cflowItems = cflowTable.getItems();
		for (int i = 0; i < cflowItems.length; i++) {
			if (cflowItems[i].getChecked()) {
				if (cflowItems[i].getData() instanceof MethodKey) {
					MethodKey meth = (MethodKey)cflowItems[i].getData();
					CFlowPointcut cflowCond = new CFlowPointcut();
					MethodExecutionPointcut mep = new MethodExecutionPointcut(meth);
					//cflowCond.setMethod(meth);
					cflowCond.setPointcut(mep);
					exceptCond.addPointcut(cflowCond);
				}
			}
		}
		
	}
	
	private void applyOnlyGroup(AndPointcut onlyCond) {
		Table cflowTable = dialog.getOnlyCFlowTable();
		TableItem[] cflowItems = cflowTable.getItems();
		for (int i = 0; i < cflowItems.length; i++) {
			if (cflowItems[i].getChecked()) {
				if (cflowItems[i].getData() instanceof MethodKey) {
					MethodKey meth = (MethodKey)cflowItems[i].getData();
					CFlowPointcut cflowCond = new CFlowPointcut();
					MethodExecutionPointcut mep = new MethodExecutionPointcut(meth);
					//cflowCond.setMethod(meth);
					cflowCond.setPointcut(mep);
					onlyCond.addPointcut(cflowCond);
				}
			}
		}
	}

	public void widgetDefaultSelected(SelectionEvent e) {
	}

	/**
	 * @return Returns the dialog.
	 */
	public ContextDialog getDialog() {
		return dialog;
	}

	/**
	 * @param dialog The dialog to set.
	 */
	public void setDialog(ContextDialog dialog) {
		this.dialog = dialog;
	}

	/**
	 * @return Returns the breakpoint.
	 */
	public IJavaContextualBreakpoint getBreakpoint() {
		return breakpoint;
	}

	/**
	 * @param breakpoint The breakpoint to set.
	 */
	public void setBreakpoint(IJavaContextualBreakpoint breakpoint) {
		this.breakpoint = breakpoint;
	}

}