package contextualbreakpoints.ui.widgets;

import java.text.MessageFormat;

import org.eclipse.jdt.internal.debug.ui.JDIDebugUIPlugin;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

import contextualbreakpoint.ui.utils.FriendlyNames;
import contextualbreakpoints.breakpoints.IJavaContextualBreakpoint;
import contextualbreakpoints.contextlogic.DefaultsLogic;

public class ClearContextDialog {
	public void open(IJavaContextualBreakpoint bp) {
		
		final Display display= JDIDebugUIPlugin.getStandardDisplay();
		if (display.isDisposed()) {
			System.out.println("display disposed?"); //$NON-NLS-1$
			return;
		}
		
		String body = Messages.getString("ClearContextDialog.0") + FriendlyNames.getFriendlyName(bp) + Messages.getString("ClearContextDialog.1") + //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ 
						FriendlyNames.getFriendlyName(bp.getContextCondition()); 
		
		String title = Messages.getString("ClippyDialog.8");  //$NON-NLS-1$
		
		ClearContextRunnable runnable = new ClearContextRunnable(display, title, body, bp);
		display.asyncExec(runnable);
	}
	
	public static ClearContextDialog getCopy() {
		return new ClearContextDialog();
	}
	
	private class ClearContextRunnable implements Runnable {

		private Display display;
		private String message;
		private String title;
		private IJavaContextualBreakpoint bp;
		
		public ClearContextRunnable (Display display, String title, String message, IJavaContextualBreakpoint bp) {
			this.display = display;
			this.message = message;
			this.title = title;
			this.bp = bp;
		}
		
		public void run() {
			if (display.isDisposed()) {
				System.out.println("display disposed?"); //$NON-NLS-1$
				return;
			}
			Shell shell= JDIDebugUIPlugin.getActiveWorkbenchShell();
			MessageDialog dialog = new MessageDialog(shell, title, null, message, MessageDialog.WARNING, new String[] {Messages.getString("ClearContextDialog.2"), Messages.getString("ClearContextDialog.3")}, 0); //$NON-NLS-1$ //$NON-NLS-2$
			int result = dialog.open();
			if (result == Window.OK) {
				// make the breakpoint like new
				bp.setContextCondition(DefaultsLogic.getDefaultResidual());
				bp.resetContext();
				bp.setHasEverSuspended(false);
			}
		}
		
	}
	
}
