package contextualbreakpoints.ui.widgets;

import java.text.MessageFormat;
import java.util.List;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.jdt.debug.core.IJavaBreakpoint;
import org.eclipse.jdt.internal.debug.ui.JDIDebugUIPlugin;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

import contextualbreakpoint.ui.utils.FriendlyNames;
import contextualbreakpoints.breakpoints.IJavaContextualBreakpoint;
import contextualbreakpoints.breakpoints.IJavaContextualLineBreakpoint;
import contextualbreakpoints.contextlogic.IClippyDialog;
import contextualbreakpoints.pointcuts.AndPointcut;
import contextualbreakpoints.pointcuts.NotPointcut;
import contextualbreakpoints.pointcuts.OrPointcut;
import contextualbreakpoints.pointcuts.Pointcut;

public class ClippyDialog implements IClippyDialog {
	
	/**
	 * @return a copy of the clippy dialog
	 */
	public static IClippyDialog getCopy() {
		return new ClippyDialog();
	}
	
	public ClippyDialog() {
		// nothing to do for now
	}
	
	private class ClippyRunnable implements Runnable {

		private Display display;
		private String message;
		private String title;
		private IJavaContextualBreakpoint bp;
		private Pointcut[] pcs;
		private int situation;
		
		public ClippyRunnable (Display display, String title, String message, 
				IJavaContextualBreakpoint bp, Pointcut[] pcs, int situation) {
			this.display = display;
			this.message = message;
			this.title = title;
			this.bp = bp;
			this.pcs = pcs;
			this.situation = situation;
		}
		
		private void addNegativeRefinement(int pcIndex) {
			
			Pointcut pc = pcs[pcIndex];
			
			// refine it based on suggestions
			System.out.println("refining on disable.."); //$NON-NLS-1$
			Pointcut cond = bp.getContextCondition();
			OrPointcut exceptCond = null;
			if (cond instanceof AndPointcut) {
				AndPointcut andCond = (AndPointcut)cond;
				List conds = andCond.getPointcuts();
				if (conds.size() == 2) {
					if (conds.get(0) instanceof NotPointcut) {
						NotPointcut notCond = (NotPointcut)conds.get(0);
						if (notCond.getPointcut() instanceof OrPointcut) {
							exceptCond = (OrPointcut)notCond.getPointcut();
						}
					}
				}
			}
			if (exceptCond != null) {
				exceptCond.addPointcut(pc);
			} else {
				System.out.println("internal error: breakpoint context not in expected form"); //$NON-NLS-1$
			}
		}
		
		private void addPositiveRefinement(int pcIndex) {
			
			Pointcut pc = pcs[pcIndex];
			
			// refine it based on suggestions
			System.out.println("refining on first suspend..."); //$NON-NLS-1$
			Pointcut cond = bp.getContextCondition();
			AndPointcut onlyCond = null;
			if (cond instanceof AndPointcut) {
				AndPointcut andCond = (AndPointcut)cond;
				List conds = andCond.getPointcuts();
				if (conds.size() == 2) {
					if (conds.get(1) instanceof AndPointcut) {
						onlyCond = (AndPointcut)conds.get(1);
					}
				}
			}
			if (onlyCond != null) {
				onlyCond.addPointcut(pc);
			} else {
				System.out.println("internal error: breakpoint context not in expected form"); //$NON-NLS-1$
			}
		}
		
		public void run() {
			if (display.isDisposed()) {
				System.out.println("display disposed?"); //$NON-NLS-1$
				return;
			}
			Shell shell= JDIDebugUIPlugin.getActiveWorkbenchShell();
			String[] selectionText = composeSelectionText(pcs);
			boolean showCancel = selectionText.length > 0;
			ClippyDialogDelegate dialog = new ClippyDialogDelegate(shell, title, message, selectionText, 0, showCancel);
			int result = dialog.open();
			if (result == Window.OK) {
				int selection = dialog.getSelected();
				if (selection < pcs.length) {
					switch (situation) {
					case BP_FIRST_BREAK:
						addPositiveRefinement(selection);
						break;
					case BP_DISABLE:
						addNegativeRefinement(selection);
						try {
							((IJavaBreakpoint)bp).setEnabled(true);
						} catch (CoreException e) {
							System.err.println("something wrong re-enabling breakpoint"); //$NON-NLS-1$
							e.printStackTrace();
						}
						break;
					case BP_RESTRICT:
						addPositiveRefinement(selection);
						break;
					case BP_EXCLUDE:
						addNegativeRefinement(selection);
						break;
					case BP_RESTRICT_ORIG:
						addPositiveRefinement(selection);
						break;
					}
				}
			}
		}
		
	}
	
	private String[] composeSelectionText(Pointcut[] pcs) {
		String[] strings = new String[pcs.length];
		for (int i = 0; i < pcs.length; i++) {
			strings[i] = FriendlyNames.getFriendlyName(pcs[i]);
		}
//		strings[pcs.length] = Messages.getString("ClippyDialog.3");  //$NON-NLS-1$
		return strings;
	}
	
	private String composeRestrictMessage(IJavaContextualBreakpoint bp, boolean na) {
		String msg = ""; //$NON-NLS-1$
		if (na) {
			msg = Messages.getString("ClippyDialog.15") + FriendlyNames.getFriendlyName(bp) + Messages.getString("ClippyDialog.16"); //$NON-NLS-1$ //$NON-NLS-2$
		} else {
			msg = Messages.getString("ClippyDialog.10") + FriendlyNames.getFriendlyName(bp) +  Messages.getString("ClippyDialog.11") +  //$NON-NLS-1$ //$NON-NLS-2$
       	 			Messages.getString("ClippyDialog.6");  //$NON-NLS-1$
		}
		return msg;
	}
	
	private String composeExcludeMessage(IJavaContextualBreakpoint bp, boolean na) {
		String msg = ""; //$NON-NLS-1$
		if (na) {
			msg = Messages.getString("ClippyDialog.18") + FriendlyNames.getFriendlyName(bp); //$NON-NLS-1$
		} else {
			msg = Messages.getString("ClippyDialog.12") + FriendlyNames.getFriendlyName(bp) + Messages.getString("ClippyDialog.13") + //$NON-NLS-1$ //$NON-NLS-2$
			Messages.getString("ClippyDialog.0"); //$NON-NLS-1$
		}
		return msg;
	}
	
	private String composeFirstBreakMessage(IJavaContextualBreakpoint bp, boolean na) {
		String msg = Messages.getString("ClippyDialog.4") + FriendlyNames.getFriendlyName(bp) + Messages.getString("ClippyDialog.5")  +    //$NON-NLS-1$ //$NON-NLS-2$
						Messages.getString("ClippyDialog.6");  //$NON-NLS-1$
		return msg;
	}
	
	private String composeDisableMessage(IJavaContextualBreakpoint bp, boolean na) {
		String msg = Messages.getString("ClippyDialog.2") + FriendlyNames.getFriendlyName(bp) +  //$NON-NLS-1$
						Messages.getString("ClippyDialog.1") +  //$NON-NLS-1$
						Messages.getString("ClippyDialog.0"); //$NON-NLS-1$
		return msg;
	}
	
	private String composeRestrictOrigMessage(IJavaContextualBreakpoint bp, boolean na) {
		String msg = ""; //$NON-NLS-1$
		if (na) {
			msg = Messages.getString("ClippyDialog.20") + FriendlyNames.getFriendlyName(bp) + Messages.getString("ClippyDialog.21"); //$NON-NLS-1$ //$NON-NLS-2$
		} else {
			msg = Messages.getString("ClippyDialog.10") + FriendlyNames.getFriendlyName(bp) +  Messages.getString("ClippyDialog.22") +  //$NON-NLS-1$ //$NON-NLS-2$ 
	 			Messages.getString("ClippyDialog.6");  //$NON-NLS-1$
		}
		return msg;
	}
	
	public void open(IJavaContextualBreakpoint bp, Pointcut[] pcs, int situation) {
		
		final Display display= JDIDebugUIPlugin.getStandardDisplay();
		if (display.isDisposed()) {
			System.out.println("display disposed?"); //$NON-NLS-1$
			return;
		}
		
		String body = ""; //$NON-NLS-1$
		switch (situation) {
		case BP_FIRST_BREAK:
			body = composeFirstBreakMessage(bp, pcs.length < 1);
			break;
		case BP_DISABLE:
			body = composeDisableMessage(bp, pcs.length < 1);
			break;
		case BP_RESTRICT:
			body = composeRestrictMessage(bp, pcs.length < 1);
			break;
		case BP_EXCLUDE:
			body = composeExcludeMessage(bp, pcs.length < 1);
			break;
		case BP_RESTRICT_ORIG:
			body = composeRestrictOrigMessage(bp, pcs.length < 1);
			break;
		}
		
		String title = Messages.getString("ClippyDialog.8");  //$NON-NLS-1$
		
		ClippyRunnable runnable = new ClippyRunnable(display, title, body, bp, pcs, situation);
		display.asyncExec(runnable);
	}
}
