package contextualbreakpoints.ui.widgets;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

/**
 * 
 * Simple radio button dialog
 *
 */
public class ClippyDialogDelegate extends Dialog {
	
	private String title;
	private String message;
	private String[] selections;
	private Button[] radios;
	private int defaultSelection;
	private int selected;
	private boolean showCancel;
	
	public ClippyDialogDelegate (Shell parent, String title, String message, String[] selections, int defaultSelection, boolean showCancel) {
		super(parent);
		this.title = title;
		this.message = message;
		this.selections = selections;
		this.defaultSelection = defaultSelection;
		this.selected = defaultSelection;
		this.showCancel = showCancel;
	}
	
	public int getSelected() {
		return selected;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createDialogArea(Composite parent) {
		getShell().setText(title);
		
		// set layout
		Composite composite = (Composite)super.createDialogArea(parent);
		Composite innerComposite = new Composite(composite, SWT.NONE);
		innerComposite.setLayoutData(new GridData());
		innerComposite.setLayout(new GridLayout());
		
		// add the message
		Label label= new Label(innerComposite, SWT.NONE);
		label.setText(message);
		label.setLayoutData(new GridData());

		// add the radios
		createRadioArea(innerComposite);
		
		applyDialogFont(composite);		
		return composite;
	}
	
	private void createRadioArea(Composite composite) {
		radios = new Button[selections.length];
		for (int i = 0; i < selections.length; i++) {
			Button button = new Button(composite, SWT.RADIO | SWT.LEFT);
	        button.setText(selections[i]);
			button.setLayoutData(new GridData());
	        button.setSelection(i==defaultSelection);
	        button.setData(new Integer(i));
	        button.addSelectionListener(new SelectionAdapter(){
	        	public void widgetSelected(SelectionEvent e) {
	        		updateSelection();
	        	}
	        });
	        radios[i] = button;
		}
	}
	
	private void updateSelection() {
		for (int i = 0; i < radios.length; i++) {
			if (radios[i].getSelection()) {
				selected = i;
			}
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createButtonsForButtonBar(org.eclipse.swt.widgets.Composite)
	 */
	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL, true); 
		if (showCancel) {
			createButton(parent, IDialogConstants.CANCEL_ID, IDialogConstants.CANCEL_LABEL, false); 
		}
	}

}
