package contextualbreakpoints.ui.widgets;

import java.util.Iterator;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.ScrollBar;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;

import contextualbreakpoint.ui.utils.FriendlyNames;
import contextualbreakpoints.breakpoints.IJavaContextualBreakpoint;
import contextualbreakpoints.breakpoints.IJavaContextualLineBreakpoint;
import contextualbreakpoints.contextlogic.DebuggerContext;
import contextualbreakpoints.contextlogic.DefaultsLogic;
import contextualbreakpoints.contextlogic.MethodKey;
import contextualbreakpoints.ui.listeners.ContextResetListener;

public class ContextDialog extends Dialog {
	Object result;
	
	private Table exceptCFlowTable;
	private Table exceptAfterTable;
	private Table exceptBeforeTable;
	
	private Table onlyCFlowTable;
	
	private Label breakpointContextExpression;
	private Composite breakpointExpressionComposite;
		
	public ContextDialog (Shell parent, int style) {
		super (parent, style);
		setText(Messages.getString("LaunchContextDialogButton.buttonCaption")); //$NON-NLS-1$
	}
	public ContextDialog (Shell parent) {
		this (parent, 0); // your default style bits go here (not the Shell's style bits)
	}
	
	private void setupExceptGroup(Group exceptGroup) {
		// setup the Except When group
		exceptGroup.setText("Breakpoint active except when");
		GridLayout exceptGridLayout = new GridLayout();
		exceptGridLayout.numColumns = 1;
		exceptGroup.setLayout(exceptGridLayout);
		
		Label cflowLabel = new Label(exceptGroup, SWT.NONE);
		cflowLabel.setText("in control flow of methods: (call stack shown)");
		
		exceptCFlowTable = new Table(exceptGroup, SWT.FULL_SELECTION | SWT.CHECK);
		exceptCFlowTable.setHeaderVisible(false);
		exceptCFlowTable.setLinesVisible(true);
		
		Label afterLabel = new Label(exceptGroup, SWT.NONE);
		afterLabel.setText("after breakpoints: (reached breakpoints shown)");
		
		exceptAfterTable = new Table(exceptGroup, SWT.FULL_SELECTION | SWT.CHECK);
		exceptAfterTable.setHeaderVisible(false);
		exceptAfterTable.setLinesVisible(true);
		
		Label beforelabel = new Label(exceptGroup, SWT.NONE);
		beforelabel.setText("before breakpoints: (unreached breakpoints shown)");
		
		exceptBeforeTable = new Table(exceptGroup, SWT.FULL_SELECTION | SWT.CHECK);
		exceptBeforeTable.setHeaderVisible(false);
		exceptBeforeTable.setLinesVisible(true);
	}
	
	private void setupOnlyGroup(Group onlyGroup) {
		// setup the Only When group
		onlyGroup.setText("Breakpoint active only when");
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 1;
		onlyGroup.setLayout(gridLayout);
		
		Label cflowLabel = new Label(onlyGroup, SWT.NONE);
		cflowLabel.setText("in control flow of methods: (method breakpoint methods shown)");
		
		onlyCFlowTable = new Table(onlyGroup, SWT.FULL_SELECTION | SWT.CHECK);
		onlyCFlowTable.setHeaderVisible(false);
		onlyCFlowTable.setLinesVisible(true);
	}
	
	private void populateGroups(IJavaContextualBreakpoint breakpoint) {
		
		// just fill in based on current context
		// I can't figure out how to make the dialog consistent if attached 
		// context can be edited
		populateExceptGroup();
		populateOnlyGroup();
	}
	
	private void populateExceptGroup() {
		DebuggerContext debugContext = DebuggerContext.getInstance();
	
		List beforeBreakpoints = debugContext.getNotReachedBreakpoints();
		for (Iterator i = beforeBreakpoints.iterator(); i.hasNext();) {
			Object obj = i.next();
			if (obj instanceof IJavaContextualLineBreakpoint) {
				IJavaContextualBreakpoint bp = (IJavaContextualBreakpoint)obj;
				TableItem item = new TableItem(exceptBeforeTable, SWT.NONE);
				item.setText(FriendlyNames.getFriendlyName(bp));
				item.setData(bp);
			}
		}
		
		List afterBreakpoints = debugContext.getReachedBreakpoints();
		for (Iterator i = afterBreakpoints.iterator(); i.hasNext();) {
			Object obj = i.next();
			if (obj instanceof IJavaContextualLineBreakpoint) {
				IJavaContextualBreakpoint bp = (IJavaContextualBreakpoint)obj;
				TableItem item = new TableItem(exceptAfterTable, SWT.NONE);
				item.setText(FriendlyNames.getFriendlyName(bp));
				item.setData(bp);
			}
		}
		
		List cflowMethods = debugContext.getCallStack();
		List cflowMethodsDefaults = DefaultsLogic.getExceptCFlowDefaults();
		for (Iterator i = cflowMethods.iterator(); i.hasNext();) {
			Object obj = i.next();
			if (obj instanceof MethodKey) {
				MethodKey meth = (MethodKey)obj;
				TableItem item = new TableItem(exceptCFlowTable, SWT.NONE);
				item.setText(meth.getClassName()+":"+meth.getMethodName()+" - " + meth.getMethodSignature());
				item.setData(meth);
				System.out.println("checking default: " + meth);
				if (cflowMethodsDefaults.contains(meth)) {
					item.setChecked(true);
				}
			}
		}
	}
	
	private void populateOnlyGroup() {
		DebuggerContext debugContext = DebuggerContext.getInstance();
		
		List cflowMethods = debugContext.getMethodBreakpointMethods();
		for (Iterator i = cflowMethods.iterator(); i.hasNext();) {
			Object obj = i.next();
			if (obj instanceof MethodKey) {
				MethodKey meth = (MethodKey)obj;
				TableItem item = new TableItem(onlyCFlowTable, SWT.NONE);
				item.setText(meth.getClassName()+":"+meth.getMethodName()+" - " + meth.getMethodSignature());
				item.setData(meth);
			}
		}
	}
	
	public void updateBreakpointContextExpression(IJavaContextualBreakpoint breakpoint) {
		String labelText = FriendlyNames.getFriendlyName(breakpoint.getContextCondition());
		breakpointContextExpression.setText(labelText);
		breakpointContextExpression.setToolTipText(labelText);
		breakpointExpressionComposite.pack();
		breakpointContextExpression.pack();
	}
	
	
	private void setupButtonComposite(Composite buttonGroup, IJavaContextualBreakpoint breakpoint) {
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		buttonGroup.setLayout(gridLayout);
		
		Button resetButton = new Button(buttonGroup, SWT.NONE);
		resetButton.setText("Clear Breakpoint Condition");
		ContextResetListener resetListener = new ContextResetListener();
		resetListener.setBreakpoint(breakpoint);
		resetListener.setDialog(this);
		resetButton.addSelectionListener(resetListener);
		
//		Button applyButton = new Button(buttonGroup, SWT.NONE);
//		applyButton.setText("Add to Breakpoint Condition");
//		ContextApplyListener applyListener = new ContextApplyListener();
//		applyListener.setBreakpoint(breakpoint);
//		applyListener.setDialog(this);
//		applyButton.addSelectionListener(applyListener);
	}
	
	private void setupBreakpointExpressionComposite(Composite comp) {
		GridLayout gridLayout = new GridLayout();
		comp.setLayout(gridLayout);
		
		Label label = new Label(comp, SWT.NONE);
		label.setText("Current breakpoint context: ");
		breakpointContextExpression = new Label(comp, SWT.SHADOW_IN);
	}
	
	public Object open (IJavaContextualBreakpoint breakpoint) {
		Shell parent = getParent();
		Shell shell = new Shell(parent, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL | SWT.H_SCROLL | SWT.V_SCROLL);
		shell.setText(getText());
		
		// Your code goes here (widget creation, set result, etc).
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 1;
		shell.setLayout(gridLayout);
		
		breakpointExpressionComposite = new Composite(shell, SWT.NONE);
		setupBreakpointExpressionComposite(breakpointExpressionComposite);
		
		
//		Label descriptionLeft = new Label(shell, SWT.NONE);
//		descriptionLeft.setText("The following elements are relevant to the current debugger context:");
//		
//		Label descriptionLeft1 = new Label(shell, SWT.NONE);
//		descriptionLeft1.setText("(the suggested context elements have been selected)");
//		
//		Group exceptGroup = new Group(shell, SWT.SHADOW_ETCHED_IN);
//		Group onlyGroup = new Group(shell, SWT.SHADOW_ETCHED_IN);
//		
//		setupExceptGroup(exceptGroup);
//		setupOnlyGroup(onlyGroup);
//		
//		populateGroups(breakpoint);
		
		Composite buttonGroup = new Composite(shell, SWT.NONE);
		setupButtonComposite(buttonGroup, breakpoint);

		
		updateBreakpointContextExpression(breakpoint);
		
		shell.pack();
		
		shell.setSize(640, 200);
		
		
		class mySelectListener extends SelectionAdapter {
			
			private Shell shell;
			private ScrollBar bar;
			private int last;
			private boolean horiz;
			
			public mySelectListener(Shell shell, ScrollBar bar, boolean horiz) {
				this.shell = shell;
				this.bar = bar;
				last = 0;
				this.horiz = horiz;
			}
			
			public void widgetSelected(SelectionEvent e) {
				int select = bar.getSelection();
				select = select * -3;
				if (horiz) {
					shell.scroll(select, 0,last,0, 640, 600, true);
				} else {
					shell.scroll(0, select, 0, last, 640, 600, true);
				}
				last = select;
			}
		}
		
		shell.getHorizontalBar().addSelectionListener(new 
				mySelectListener(shell, shell.getHorizontalBar(), true));
		shell.getVerticalBar().addSelectionListener(new
				mySelectListener(shell, shell.getVerticalBar(), false));
		
		shell.open();
		


		
		Display display = parent.getDisplay();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) display.sleep();
		}
		return result;
	}

	/**
	 * @return Returns the exceptCFlowTable.
	 */
	public Table getExceptCFlowTable() {
		return exceptCFlowTable;
	}
	/**
	 * @return Returns the exceptAfterTable.
	 */
	public Table getExceptAfterTable() {
		return exceptAfterTable;
	}
	/**
	 * @return Returns the exceptBeforeTable.
	 */
	public Table getExceptBeforeTable() {
		return exceptBeforeTable;
	}
	
	public Table getOnlyCFlowTable() {
		return onlyCFlowTable;
	}
	
//	private void oldpopulateExceptGroup(OrCondition exceptChildCond) {
//		List conds = exceptChildCond.getConditions();
//		for (Iterator i = conds.iterator(); i.hasNext();) {
//			Condition cond = (Condition)i.next();
//			if (cond instanceof CFlowCondition) {
//				IMethod method = ((CFlowCondition)cond).getMethod();
//				TableItem item = new TableItem(exceptCFlowTable, SWT.NONE);
//				item.setText(method.toString());
//			} else if (cond instanceof AfterCondition) {
//				IJavaContextualLineBreakpoint bp = ((AfterCondition)cond).getBreakpoint();
//				TableItem item = new TableItem(exceptAfterTable, SWT.NONE);
//				item.setText(bp.toString());
//			} else if (cond instanceof BeforeCondition) {
//				IJavaContextualLineBreakpoint bp = ((BeforeCondition)cond).getBreakpoint();
//				TableItem item = new TableItem(exceptBeforeTable, SWT.NONE);
//				item.setText(bp.toString());
//			} 
//		}
//	}
//	
//	private void oldpopulateOnlyGroup(AndCondition onlyCond) {
//		List conds = onlyCond.getConditions();
//		for (Iterator i = conds.iterator(); i.hasNext();) {
//			Condition cond = (Condition)i.next();
//			if (cond instanceof CFlowCondition) {
//				IMethod method = ((CFlowCondition)cond).getMethod();
//				TableItem item = new TableItem(onlyCFlowTable, SWT.NONE);
//				item.setText(method.toString());
//			} else if (cond instanceof AfterCondition) {
//				IJavaContextualLineBreakpoint bp = ((AfterCondition)cond).getBreakpoint();
//				TableItem item = new TableItem(onlyAfterTable, SWT.NONE);
//				item.setText(bp.toString());
//			} else if (cond instanceof BeforeCondition) {
//				IJavaContextualLineBreakpoint bp = ((BeforeCondition)cond).getBreakpoint();
//				TableItem item = new TableItem(onlyBeforeTable, SWT.NONE);
//				item.setText(bp.toString());
//			} 
//		}
//	}
 }