package contextualbreakpoints.breakpoints;

import java.util.List;

import contextualbreakpoints.pointcuts.Pointcut;

public interface IJavaContextualBreakpoint {
	
	/**
	 * @return true if this breakpoint never suspends
	 */
	public boolean isShadowOnly();
	
	/**
	 * @param yes true if this breakpoint should never suspend
	 */
	public void setShadowOnly(boolean yes);
	
	/**
	 * 
	 */
	public void resetContext();
	
	/**
	 * @param condition the NotCondition to set
	 */
	public void setContextCondition(Pointcut condition);
	
	/**
	 * @return the NotCondtion
	 */
	public Pointcut getContextCondition();
	
	/**
	 * @return true if the breakpoint has been reached at least once
	 */
	public boolean hasBeenReached();
	
	public void setHasBeenReached(boolean reached);
	
	/**
	 * 
	 * @return true if the breakpoint has <i>suspended</i> at least once
	 */
	public boolean hasSuspended();
	
	/**
	 * 
	 * @return true iff the breakpoint has <i>ever</i> suspended since creation
	 */
	public boolean hasEverSuspended();
	
	
	/**
	 * @param suspended 
	 */
	public void setHasEverSuspended(boolean suspended);
	
	/**
	 * @param pc the context history of contexts encountered by this bp
	 */
	public void setSavedContext(List pc);
	
	/**
	 * @return history of saved contexts in order encountered by this bp
	 */
	public List getSavedContext();
	
	/**
	 * @param methodKeys the call stack associated with this bp
	 */
	public void setSavedCallStack(List methodKeys);
	
	/**
	 * @return the call stack associated with this bp
	 */
	public List getSavedCallStack();
	
	/**
	 * @param bps breakpoints not been reached 
	 */
	public void setSavedBeforeBreakpoints(List bps);
	
	/**
	 * @return breakpoints that weren't reached
	 */
	public List getSavedBeforeBreakpoints();
	
	/**
	 * @param bps breakpoints that have been reached
	 */
	public void setSavedAfterBreakpoints(List bps);
	
	/**
	 * @return breakpoints that were reached
	 */
	public List getSavedAfterBreakpoints();
	
}
