package contextualbreakpoints.breakpoints;


import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jdt.internal.debug.core.breakpoints.JavaMethodBreakpoint;
import org.eclipse.jdt.internal.debug.core.model.JDIDebugTarget;
import org.eclipse.jdt.internal.debug.core.model.JDIThread;

import com.sun.jdi.event.Event;

import contextualbreakpoints.contextlogic.DefaultsLogic;
import contextualbreakpoints.pointcuts.Pointcut;


/**
 * @author Rick
 * Just like a JavaMethodBreakpoint except it returns
 */
/**
 * @author Rick
 *
 */
public class JavaContextualMethodBreakpoint extends JavaMethodBreakpoint implements IJavaContextualMethodBreakpoint {
	
	protected Pointcut contextCondition;
	
	private boolean isTriggerOnly;
	
	private boolean hasBeenReached;
	
	private boolean hasSuspended;
	
	private boolean hasEverSuspended;
	
	// saved contexts
	private List savedContexts;
	private List savedCallStack;
	private List savedBeforeBps;
	private List savedAfterBps;
	
	public JavaContextualMethodBreakpoint(IResource resource, String typePattern, String methodName, String methodSignature, boolean entry, boolean exit, boolean nativeOnly, int lineNumber, int charStart, int charEnd, int hitCount, boolean register, Map attributes) throws CoreException {
		super(resource, typePattern, methodName, methodSignature, entry, exit, nativeOnly, lineNumber, charStart, charEnd, hitCount, register, attributes);
		
		contextCondition = DefaultsLogic.getDefaultResidual();
		setShadowOnly(false);
		setHasEverSuspended(false);
		savedContexts = new ArrayList();
		resetContext();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jdt.internal.debug.core.breakpoints.JavaBreakpoint#handleBreakpointEvent(com.sun.jdi.event.Event, org.eclipse.jdt.internal.debug.core.model.JDIDebugTarget, org.eclipse.jdt.internal.debug.core.model.JDIThread)
	 */
	public boolean handleBreakpointEvent(Event event, JDIDebugTarget target, JDIThread thread) {
		hasBeenReached = true;
		
		// whether to suspend depends only on 
		if ((contextCondition.isMatch(thread) && !isTriggerOnly)) {
			if (super.handleBreakpointEvent(event, target, thread)) {
				return true;
			}
			hasSuspended = true;
			return false;
		}
		return true;
	}

	/**
	 * @return Returns the isTriggerOnly.
	 */
	public boolean isShadowOnly() {
		return isTriggerOnly;
	}

	/**
	 * @param isTriggerOnly The isTriggerOnly to set.
	 */
	public void setShadowOnly(boolean isTriggerOnly) {
		this.isTriggerOnly = isTriggerOnly;
	}

	
	public Pointcut getContextCondition() {
		return contextCondition;
	}

	public void setContextCondition(Pointcut contextCondition) {
		this.contextCondition = contextCondition;
	}
	
	/**
	 * resets the breakpoint for conditional structures
	 */
	public void resetContext() {
		hasBeenReached = false;
		hasSuspended = false;
	}

	public boolean hasBeenReached() {
		return hasBeenReached;
	}

	public boolean hasSuspended() {
		return hasSuspended;
	}
	
	public boolean hasEverSuspended() {
		return hasEverSuspended;
	}
	
	public void setHasEverSuspended(boolean b) {
		hasEverSuspended = b;
	}

	public void setHasBeenReached(boolean reached) {
		hasBeenReached = reached;
		
	}

	public void setSavedContext(List pc) {
		savedContexts = pc;
	}

	public List getSavedContext() {
		return savedContexts;
	}
	
	public String toString() {
		try {
			return "JavaContextualMethodBreakpoint: " + getTypeName() + " : " + getMethodName() + getMethodSignature(); //$NON-NLS-1$ //$NON-NLS-2$
		} catch (CoreException e) {
			return super.toString();
		}
	}

	/**
	 * @return Returns the savedCallStack.
	 */
	public List getSavedCallStack() {
		return savedCallStack;
	}

	/**
	 * @param savedCallStack The savedCallStack to set.
	 */
	public void setSavedCallStack(List savedCallStack) {
		this.savedCallStack = savedCallStack;
	}
	
	/* (non-Javadoc)
	 * @see contextualbreakpoints.breakpoints.IJavaContextualBreakpoint#setSavedBeforeBreakpoints(java.util.List)
	 */
	public void setSavedBeforeBreakpoints(List bps) {
		savedBeforeBps = bps;
	}


	/* (non-Javadoc)
	 * @see contextualbreakpoints.breakpoints.IJavaContextualBreakpoint#getSavedBeforeBreakpoints()
	 */
	public List getSavedBeforeBreakpoints() {
		return savedBeforeBps;
	}


	/* (non-Javadoc)
	 * @see contextualbreakpoints.breakpoints.IJavaContextualBreakpoint#setSavedAfterBreakpoints(java.util.List)
	 */
	public void setSavedAfterBreakpoints(List bps) {
		savedAfterBps = bps;
	}


	public List getSavedAfterBreakpoints() {
		return savedAfterBps;
	}
}
