package contextualbreakpoints.contextlogic;

import java.util.ArrayList;
import java.util.EmptyStackException;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.Stack;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.IBreakpointManager;
import org.eclipse.debug.core.model.IBreakpoint;
import org.eclipse.debug.core.model.IDebugTarget;
import org.eclipse.jdt.debug.core.IJavaBreakpoint;
import org.eclipse.jdt.debug.core.IJavaLineBreakpoint;
import org.eclipse.jdt.internal.debug.core.EventDispatcher;
import org.eclipse.jdt.internal.debug.core.IJDIEventListener;
import org.eclipse.jdt.internal.debug.core.model.JDIDebugTarget;
import org.eclipse.jdt.internal.debug.core.model.JDIThread;

import com.sun.jdi.IncompatibleThreadStateException;
import com.sun.jdi.Location;
import com.sun.jdi.Method;
import com.sun.jdi.ReferenceType;
import com.sun.jdi.StackFrame;
import com.sun.jdi.ThreadReference;
import com.sun.jdi.VirtualMachine;
import com.sun.jdi.event.BreakpointEvent;
import com.sun.jdi.event.Event;
import com.sun.jdi.event.LocatableEvent;
import com.sun.jdi.request.BreakpointRequest;
import com.sun.jdi.request.EventRequest;

import contextualbreakpoints.breakpoints.IJavaContextualBreakpoint;
import contextualbreakpoints.breakpoints.IJavaContextualLineBreakpoint;
import contextualbreakpoints.breakpoints.IJavaContextualMethodBreakpoint;
import contextualbreakpoints.breakpoints.JavaContextualLineBreakpoint;
import contextualbreakpoints.breakpoints.JavaContextualMethodBreakpoint;
import contextualbreakpoints.pointcuts.LineExecutionPointcut;
import contextualbreakpoints.pointcuts.Pointcut;
import contextualbreakpoints.pointcuts.PointcutManager;



public class DebuggerContext {
	private static DebuggerContext instance;
	private JDIThread mostRecentlySuspended;
	private JDIDebugTarget mostRecentTarget;
	private Stack mostRecentlyReachedBreakpoints;
	private LineEventCache lineEventCache;
	private boolean debuggerActive;
	private IClippyDialog clippy;
	
	private DebuggerContext() {
		instance = null;
		lineEventCache = new LineEventCache();
		debuggerActive = false;
		mostRecentlyReachedBreakpoints = new Stack();
	}
	
	public static DebuggerContext getInstance() {
		if (instance == null) {
			instance = new DebuggerContext();
		}
		return instance;
	}
	
	public List getReachedBreakpoints() {
		ArrayList list = new ArrayList();
		IBreakpoint[] bps = DebugPlugin.getDefault().getBreakpointManager().getBreakpoints();
		for (int i = 0; i < bps.length; i++) {
			if (bps[i] instanceof IJavaContextualBreakpoint) {
				IJavaContextualBreakpoint contextualBreakpoint = (IJavaContextualBreakpoint)bps[i];
				if (contextualBreakpoint.hasBeenReached()) {
					list.add(contextualBreakpoint);
				}
			}
		}
		return list;
	}
	
	public List getNotReachedBreakpoints() {
		ArrayList list = new ArrayList();
		IBreakpoint[] bps = DebugPlugin.getDefault().getBreakpointManager().getBreakpoints();
		for (int i = 0; i < bps.length; i++) {
			if (bps[i] instanceof IJavaContextualBreakpoint) {
				IJavaContextualBreakpoint contextualBreakpoint = (IJavaContextualBreakpoint)bps[i];
				if (!contextualBreakpoint.hasBeenReached()) {
					list.add(contextualBreakpoint);
				}
			}
		}
		return list;
	}
	
	public List getHasSuspendedBreakpoints() {
		ArrayList list = new ArrayList();
		IBreakpoint[] bps = DebugPlugin.getDefault().getBreakpointManager().getBreakpoints();
		for (int i = 0; i < bps.length; i++) {
			if (bps[i] instanceof IJavaContextualBreakpoint) {
				IJavaContextualBreakpoint contextualBreakpoint = (IJavaContextualBreakpoint)bps[i];
				if (contextualBreakpoint.hasSuspended()) {
					list.add(contextualBreakpoint);
				}
			}
		}
		return list;
	}
	
	public List getNotHasSuspendedBreakpoints() {
		ArrayList list = new ArrayList();
		IBreakpoint[] bps = DebugPlugin.getDefault().getBreakpointManager().getBreakpoints();
		for (int i = 0; i < bps.length; i++) {
			if (bps[i] instanceof IJavaContextualBreakpoint) {
				IJavaContextualBreakpoint contextualBreakpoint = (IJavaContextualBreakpoint)bps[i];
				if (!contextualBreakpoint.hasSuspended()) {
					list.add(contextualBreakpoint);
				}
			}
		}
		return list;
	}
	
	public Method getMethodByName(MethodKey  methKey) {
		String className = methKey.getClassName();
		String name = methKey.getMethodName();
		String signature = methKey.getMethodSignature();
		
		IDebugTarget[] targets = DebugPlugin.getDefault().getLaunchManager().getDebugTargets();
		System.out.println("there are " + targets.length + " debug targets");
		for (int i = 0; i < targets.length; i++) {
			if (targets[i] instanceof JDIDebugTarget) {
				
				JDIDebugTarget target = (JDIDebugTarget)targets[i];
				VirtualMachine vm = target.getVM();
				if (vm != null) {
					List classes = vm.allClasses();
					for (Iterator j = classes.iterator(); j.hasNext();) {
						ReferenceType ref = (ReferenceType)j.next();
						if (ref.name().equals(className)) {
							List methods = ref.allMethods();
							for (Iterator k = methods.iterator(); k.hasNext();) {
								Method toCheck = (Method)k.next();
								if (name.equals(toCheck.name()) &&
										signature.equals(toCheck.signature())) {
									return toCheck;
								}
							}
						}
					}
				}
			}
		}
		return null;
	}
	
	/**
	 * @return list of MethodKeys: class name, method name, method signature
	 */
	public List getMethodBreakpointMethods() {
		ArrayList list = new ArrayList();
		IBreakpoint[] bps = DebugPlugin.getDefault().getBreakpointManager().getBreakpoints();
		for (int i = 0; i < bps.length; i++) {
			if (bps[i] instanceof IJavaContextualMethodBreakpoint) {
				IJavaContextualMethodBreakpoint methodBreakpoint = (IJavaContextualMethodBreakpoint)bps[i];
				MethodKey methKey = new MethodKey();
				try {
					String className = methodBreakpoint.getMarker().getResource().getName();
					className = className.split("\\.")[0]; //$NON-NLS-1$
					
					//String[] classparts = className.split("\\.");
					//System.out.println("length of class parts " + classparts.length);
					methKey.setClassName(className);
					methKey.setMethodName(methodBreakpoint.getMethodName());
					methKey.setMethodSignature(methodBreakpoint.getMethodSignature());
				} catch (CoreException e) {
					e.printStackTrace();
				}
				list.add(methKey);
			}
		}
		return list;
	}
	
	/**
	 * @return list of MethodKeys, with first element == top of stack
	 */
	public List getCallStack() {
		ArrayList stack = new ArrayList();
		if (mostRecentlySuspended != null ) {
			try {
				ThreadReference uThread = mostRecentlySuspended.getUnderlyingThread();
				for (int i = 0; i < uThread.frameCount(); i++) {
					StackFrame frame = uThread.frame(i);
					Method method = frame.location().method();
					MethodKey methKey = new MethodKey();
					methKey.setClassName(method.declaringType().name());
					methKey.setMethodName(method.name());
					methKey.setMethodSignature(method.signature());
					stack.add(methKey);
				}
			} catch (IncompatibleThreadStateException e) {
				//e.printStackTrace();
				System.out.println("probably thread not suspended");
			}
		}
		return stack;
	} 

	/**
	 * @return Breakpoint corresponding to line (numbered from 0). Only returns
	 *         breakpoints currently registered in breakpoint manager, and are
	 *         contextual breakpoints
	 *         Returns null if no matching breakpoint for line
	 */
	public IJavaBreakpoint getBreakpointByLine(int line) {
		IBreakpoint[] bps = DebugPlugin.getDefault().getBreakpointManager().getBreakpoints();
		for (int i = 0; i < bps.length; i++) {
			if (bps[i] instanceof IJavaBreakpoint) {
				IJavaBreakpoint mbp = (IJavaBreakpoint)bps[i];
				IMarker marker = mbp.getMarker();
				if (marker != null) {
					try {
						Integer markerLine = (Integer)marker.getAttribute(IMarker.LINE_NUMBER);
						// markers are numbered from 1
						if (markerLine.intValue() == line + 1) {
							return mbp;
						}
					} catch (CoreException e) {
						// probably no line, don't consider this breakpoint
					} catch (ClassCastException e) {
						// probably no line, don't consider this breakpoint
					}
				}
					
			}
		}
		return null;
	}
	
	
	/**
	 * @param eventDispatcher
	 * @param target
	 * @param active true iff debugging session is going to be active
	 */
	public void reset(EventDispatcher eventDispatcher, JDIDebugTarget target, boolean active) {
		
		// reset pointcuts
		resetPointcuts();
		
		// reset breakpoints
		resetBreakpoints();
		
		// clear the line event cache
		if (active) {
			lineEventCache = new LineEventCache();
		}
		
		// and re-register all pointcut origins
		if (active) {
			registerPointcutOrigins(target);
		}
		
		mostRecentlySuspended = null;
		
//		mostRecentlyReachedBreakpoint = null;
		mostRecentlyReachedBreakpoints.clear();
		
		setDebuggerActive(active);
	}
	
	/**
	 * Set all LineExecutions to not been reached
	 */
	private void resetPointcuts() {
		Set leps = PointcutManager.getInstance().getLineExecutionPointcuts();
		if (leps != null) {
			for (Iterator i = leps.iterator(); i.hasNext();) {
				LineExecutionPointcut lep = (LineExecutionPointcut)(i.next());
				
				// all LineExecutionPointcuts should not have been executed
				lep.setBeenReached(false);
			}
		}
	}
	
	/**
	 * Set all line breakpoints to not been reached
	 */
	private void resetBreakpoints() {
		IBreakpointManager mgr = DebugPlugin.getDefault().getBreakpointManager();
		IBreakpoint[] bps = mgr.getBreakpoints();
		for (int i = 0; i < bps.length; i++) {
			if (bps[i] instanceof IJavaContextualBreakpoint) {
				IJavaContextualBreakpoint bp = (IJavaContextualBreakpoint)bps[i];
				bp.resetContext();
			}
		}
	}
	
	
	/**
	 * Re-registers all breakpoints that are origins for pointcuts, if breakpoint
	 * is shadow only (has been deleted)
	 */
	public void registerPointcutOrigins(JDIDebugTarget target) {
		if (target != null) {
			Set leps = PointcutManager.getInstance().getLineExecutionPointcuts();
			if (leps != null) {
				for (Iterator i = leps.iterator(); i.hasNext();) {
					LineExecutionPointcut lep = (LineExecutionPointcut)(i.next());

					// re-register event with VM
					try {
						IJavaContextualLineBreakpoint bp = lep.getOriginBreakpoint();
						if (bp.isShadowOnly()) {
							bp.addToTarget(target);
						}
					} catch (CoreException e) {
						System.out.println("Can't add shadow bp to target - probably debugging session ended");
						//e.printStackTrace();
					} 
					
				}
			}
		}
	}
	
	/**
	 * 
	 * @param event
	 * @param target
	 */
	public void eavesdropOnEvent(Event event, JDIDebugTarget target) {
		
//		System.out.println("eavesdropping on event " + event);
		
		if (target != null) {
			setMostRecentTarget(target);
		}
		
		// we're interested in the latest thread that suspended
		updateLatestThread(event, target);
		
		// see if any pointcuts are interested
		matchPointcuts(event);
		
		// also...maybe we can refine the context further positively?
		refinePositiveContext(event);
		
		// NOTE this should come last
		updateMostRecentBreakpoint(event);
	}
	
	
	
	private void updateMostRecentBreakpoint(Event event) {
		if (event instanceof BreakpointEvent) {
			BreakpointEvent bpEvent = (BreakpointEvent)event;
			IJavaContextualBreakpoint bp = getBreakpointByEvent(bpEvent);
			if (bp instanceof IJavaContextualLineBreakpoint) {
				IJavaContextualLineBreakpoint lineBp = (IJavaContextualLineBreakpoint)bp;
				//setMostRecentlyReachedBreakpoint(bp);
				//getMostRecentlyReachedBreakpoints().push(bp);
				if (!lineBp.fakeHandleBreakpointEvent(event, getMostRecentTarget(), mostRecentlySuspended)) {
					getMostRecentlyReachedBreakpoints().push(bp);
				}
			}
		}
	}

	/**
	 * @param event
	 * @return breakpoint <i>registered with BreakpointManager</i> by the breakpoint event
	 */
	public IJavaContextualBreakpoint getBreakpointByEvent(BreakpointEvent event) {
		IBreakpointManager bpMgr = DebugPlugin.getDefault().getBreakpointManager();
		IBreakpoint[] bps = bpMgr.getBreakpoints();
		
		// we look in the cache for the associated breakpoint
		for (int i = 0; i < bps.length; i++) {
			// note here that we assume any listener that is:
			// - an JavaContextualBreakpoint, and
			// - is registered in breakpoint manager 
			// must be the breakpoint associated with this breakpoint event
			if (bps[i] instanceof JavaContextualLineBreakpoint) {
				JavaContextualLineBreakpoint lineBps = (JavaContextualLineBreakpoint)bps[i];
				if (lineEventCache.hasRequest(lineBps, event.request())) {
					return lineBps;
				}
			} else if (bps[i] instanceof JavaContextualMethodBreakpoint) {
				JavaContextualMethodBreakpoint methBps = (JavaContextualMethodBreakpoint)bps[i];
				if (lineEventCache.hasRequest(methBps, event.request())) {
					return methBps;
				}
			}
		}
		return null;
	}

	public void refinePositiveContext(Event event) {
		if (event instanceof BreakpointEvent) {
			BreakpointEvent bpEvent = (BreakpointEvent)event;
			// get the breakpoint we want to refine
			IJavaContextualBreakpoint bp = getBreakpointByEvent(bpEvent);
			// could be something else like a watchpoint
			if (bp != null) {
				if (!bp.hasEverSuspended()) {
					// first time we suspend
					bp.setHasEverSuspended(true);

					// set the suggested pointcuts for this breakpoint
					Pointcut[] pcs = DefaultsLogic.getDefaultPositivePointcuts(bp);
					
					ArrayList list = new ArrayList();
					for (int i = 0; i < pcs.length; i++) {
						list.add(pcs[i]);
					}
					System.out.println("saving context for " + bp);
					bp.setSavedContext(list);
					
					// and save the call stack as well
					List methKeys = getCallStack();
					bp.setSavedCallStack(methKeys);
					
					// and save the before and after breakpoints
					List beforebps = getNotReachedBreakpoints();
					List afterbps = getReachedBreakpoints();
					bp.setSavedAfterBreakpoints(afterbps);
					bp.setSavedBeforeBreakpoints(beforebps);
				}
			}
		}
	}

	private void matchPointcuts(Event event) {
		if (event instanceof BreakpointEvent) {
			BreakpointEvent bpEvent = (BreakpointEvent)event;
			if (bpEvent != null) {
				Set leps = PointcutManager.getInstance().getLineExecutionPointcuts();
				if (leps != null) {
					for (Iterator i = leps.iterator(); i.hasNext();) {
						LineExecutionPointcut lep = (LineExecutionPointcut)(i.next());
						
						// check if locations are the same
						if (lep.isMatchLocation(bpEvent.location())) {
							lep.setBeenReached(true);
						}
					}
				}
			}
		}
	}

	private void updateLatestThread(Event event, JDIDebugTarget target) {
		if (event instanceof LocatableEvent) {
			LocatableEvent locatableEvent = (LocatableEvent)event;
			if (locatableEvent != null) {
				ThreadReference threadRef= locatableEvent.thread();
				if (threadRef != null) {
					JDIThread thread= target.findThread(threadRef);
					// thread.isSuspended always returns false for some reason...
					if (thread != null && !thread.isIgnoringBreakpoints()) {
						mostRecentlySuspended = thread;
					}
				}
			}
		}
	}
	
	public void eavesdropOnEventListenerAdd(IJDIEventListener listener, EventRequest request, JDIDebugTarget target) {
		//System.err.println("adding event: " + listener + " : " + request + " : " + target);
		
		// we'll keep a history of all added line breakpoint events
		if (request instanceof BreakpointRequest) {
			if (listener instanceof IJavaLineBreakpoint) {
//				System.out.println("adding request " + listener + "  :  " + request + "  :  " + target);
				lineEventCache.addEntry(listener, request, target);
			}
		}
	}

	/**
	 * @return Returns the mostRecentTarget.
	 */
	public JDIDebugTarget getMostRecentTarget() {
		return mostRecentTarget;
	}

	/**
	 * @param mostRecentTarget The mostRecentTarget to set.
	 */
	public void setMostRecentTarget(JDIDebugTarget mostRecentTarget) {
		this.mostRecentTarget = mostRecentTarget;
	}
	
	public void setDebuggerActive(boolean active) {
		debuggerActive = active;
	}
	
	public boolean getDebuggerActive() {
		return debuggerActive;
	}

	/**
	 * @return Returns the mostRecentlyReachedBreakpoint.
	 */
	public IJavaContextualLineBreakpoint getMostRecentlyReachedBreakpoint() {
		try {
			return (IJavaContextualLineBreakpoint)getMostRecentlyReachedBreakpoints().peek();
		} catch (EmptyStackException e) {
			return null;
		}
	}
	
	/**
	 * @return the most recently reached breakpoints in order
	 */
	public Stack getMostRecentlyReachedBreakpoints() {
		return mostRecentlyReachedBreakpoints;
	}

	/**
	 * @return Returns the clippy.
	 */
	public IClippyDialog getClippy() {
		return clippy;
	}

	/**
	 * @param clippy The clippy to set.
	 */
	public void setClippy(IClippyDialog clippy) {
		this.clippy = clippy;
	}

	public Location[] getLocations(IJavaContextualLineBreakpoint bp) {
		ArrayList locs = new ArrayList();
		List reqs = lineEventCache.getEventRequests(bp);
		if (reqs != null) {
			for (Iterator i = reqs.iterator(); i.hasNext();) {
				EventRequest req = (EventRequest)i.next();
				if (req instanceof BreakpointRequest) {
					BreakpointRequest bpReq = (BreakpointRequest)req;
					locs.add(bpReq.location());
				}
			}
		}
		Location[] arrLocs = new Location[locs.size()];
		for (int i = 0; i < arrLocs.length; i++) {
			arrLocs[i] = (Location)locs.get(i);
		}
		return arrLocs;
	}

}
