package contextualbreakpoints.contextlogic;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Stack;

import contextualbreakpoints.breakpoints.IJavaContextualBreakpoint;
import contextualbreakpoints.breakpoints.IJavaContextualLineBreakpoint;
import contextualbreakpoints.pointcuts.AfterPointcut;
import contextualbreakpoints.pointcuts.AndPointcut;
import contextualbreakpoints.pointcuts.CFlowPointcut;
import contextualbreakpoints.pointcuts.LineExecutionPointcut;
import contextualbreakpoints.pointcuts.MethodExecutionPointcut;
import contextualbreakpoints.pointcuts.NotPointcut;
import contextualbreakpoints.pointcuts.OrPointcut;
import contextualbreakpoints.pointcuts.Pointcut;

public class DefaultsLogic {
	
	
	/**
	 * @return List of MethodKeys
	 */
	public static List getExceptCFlowDefaults() {
		List stack = DebuggerContext.getInstance().getCallStack();
		ArrayList suggestions = new ArrayList();
		if (stack.size() > 1) {
			// 2nd item is probably the most specific we can get
			suggestions.add(stack.get(1));
		}
		return suggestions;
	}
	
	/**
	 * @return List of MethodKeys
	 */
	public static List getOnlyCFlowDefaults() {
		List stack = DebuggerContext.getInstance().getCallStack();
		ArrayList suggestions = new ArrayList();
		if (stack.size() > 1) {
			// 2nd item is probably the most specific we can get
			suggestions.add(stack.get(1));

		}
		return suggestions;
	}
	
//	/**
//	 * @param bp
//	 * @return origin breakpoint to construct LineExecution PC from
//	 */
//	private static IJavaContextualLineBreakpoint onlyAfterResolveUp(IJavaContextualLineBreakpoint bp) {
//		Pointcut cond = bp.getContextCondition();
//		AndPointcut onlyCond = null;
//		if (cond instanceof AndPointcut) {
//			AndPointcut andCond = (AndPointcut)cond;
//			List conds = andCond.getPointcuts();
//			if (conds.size() == 2) {
//				if (conds.get(1) instanceof AndPointcut) {
//					onlyCond = (AndPointcut)conds.get(1);
//				}
//			}
//		}
//		if (onlyCond != null) {
//			for (Iterator i = onlyCond.getPointcuts().iterator(); i.hasNext();) {
//				Pointcut p = (Pointcut)i.next();
//				if (p instanceof AfterPointcut) {
//					Pointcut c = ((AfterPointcut)p).getPointcut();
//					if (c instanceof LineExecutionPointcut) {
//						LineExecutionPointcut lep = (LineExecutionPointcut)c;
//						if (lep.getOriginBreakpoint().isShadowOnly()) {
//							// reached a shadow line, don't go up any further
//							return lep.getOriginBreakpoint();
//						}
//						// go up further
//						return onlyAfterResolveUp(lep.getOriginBreakpoint());
//					}
//				}
//			}
//		}
//		return bp;
//	}
	
	/**
	 * @return List of IJavaContextualLineBreakpoints
	 */
	public static List getOnlyAfterDefaults() {
		ArrayList suggestions = new ArrayList();
		// probably only after the most recently reached bp
		IJavaContextualLineBreakpoint bp = DebuggerContext.getInstance().getMostRecentlyReachedBreakpoint();
	
		if (bp != null) {
			suggestions.add(bp);
		}
		return suggestions;
	}
	
	/**
	 * @param current the current breakpoint that is suspended
	 * @return only after defaults, taking into account the current breakpoint
	 */
	public static List getOnlyAfterDefaults(IJavaContextualLineBreakpoint current) {
		ArrayList suggestions = new ArrayList();
		Stack recentBps = DebuggerContext.getInstance().getMostRecentlyReachedBreakpoints();
		
		if (recentBps.size() > 0) {
			if (recentBps.peek().equals(current)) {
				if (recentBps.size() > 1) {
					IJavaContextualLineBreakpoint bp = (IJavaContextualLineBreakpoint)(recentBps.get(recentBps.size() - 2));
					suggestions.add(bp);
				}
			} else {
				IJavaContextualLineBreakpoint bp = (IJavaContextualLineBreakpoint)(recentBps.peek());
				suggestions.add(bp);
			}
		}
		return suggestions;
	}
	
	/**
	 * @return Empty pointcut in structure expected by Contextual Breakpoints Debugger
	 */
	public static Pointcut getDefaultResidual() {
		AndPointcut condition = new AndPointcut();
		NotPointcut notCond = new NotPointcut();
		notCond.setPointcut(new OrPointcut());
		condition.addPointcut(notCond);
		condition.addPointcut(new AndPointcut());
		return condition;
	}
	
	public static boolean isDefaultResidual(Pointcut pc) {
		if (pc instanceof AndPointcut) {
			AndPointcut andPC = (AndPointcut)pc;
			if (andPC.getPointcuts().size() == 2) {
				Pointcut first = (Pointcut)andPC.getPointcuts().get(0);
				Pointcut second = (Pointcut)andPC.getPointcuts().get(1);
				if (first instanceof NotPointcut) {
					NotPointcut notPC = (NotPointcut)first;
					if (notPC.getPointcut() instanceof OrPointcut) {
						OrPointcut orPC = (OrPointcut)notPC.getPointcut();
						if (orPC.getPointcuts().size() == 0) {
							if (second instanceof AndPointcut) {
								AndPointcut andPC2 = (AndPointcut)second;
								if (andPC2.getPointcuts().size() == 0) {
									return true;
								}
							}
						}
					}
				}
			}
		}
		return false;
	}
	
	
	/**
	 * @return Array of pointcuts.  Should be individually added to an NOT(OR(...)) pointcut
	 */
	public static Pointcut[] getDefaultNegativePointcuts() {
		// default pointcuts are:
		// 1. CFLOW(MethodExecution<most specific method on stack>)
		//
		List defaultPCs = new ArrayList();
		
		// Construct 1.
		List notCFlow = getExceptCFlowDefaults();
		if (notCFlow != null) {
			for (Iterator i = notCFlow.iterator(); i.hasNext();) {
				MethodKey methKey = (MethodKey)i.next();
				MethodExecutionPointcut methPC = new MethodExecutionPointcut(methKey);
				CFlowPointcut cflowPC = new CFlowPointcut(methPC);
				defaultPCs.add(cflowPC);
			}
		}
		return castToArray(defaultPCs);
	}

	
	/**
	 * @return Array of pointcuts.  Should be individually added to an AND(...) pointcut
	 */
	public static Pointcut[] getDefaultPositivePointcuts(IJavaContextualBreakpoint current) {
		// default pointcuts are:
		// 1. AND(AFTER(LineExecution<most recent breakpoint reached>), context condition of most recent bp reached)
		// 2. CFLOW(MethodExecution<most specific method on stack>))
		//
		List defaultPCs = new ArrayList();
		
		// Construct 1.
		List onlyAfter = null;
		if (current instanceof IJavaContextualLineBreakpoint) {
			onlyAfter = getOnlyAfterDefaults((IJavaContextualLineBreakpoint)current);
		} else {
			onlyAfter  = getOnlyAfterDefaults();
		}
		if (onlyAfter != null) {
			for (Iterator i = onlyAfter.iterator(); i.hasNext();) {
				IJavaContextualLineBreakpoint bp = (IJavaContextualLineBreakpoint)i.next();

				// we allow for two options 
				//
				// FIRST OPTION:
				if (bp.getSavedContext().size() > 0) {
					reconstructLinePointcut(defaultPCs, bp, (Pointcut)(bp.getSavedContext().get(0)));
				}
				
				// SECOND OPTION:
				reconstructLinePointcut(defaultPCs, bp, bp.getContextCondition());
			}
		}
		
		// Construct 2.
		List onlyCFlow = getOnlyCFlowDefaults();
		if (onlyCFlow != null) {
			for (Iterator i = onlyCFlow.iterator(); i.hasNext();) {
				MethodKey mKey = (MethodKey)i.next();
				MethodExecutionPointcut methPC = new MethodExecutionPointcut(mKey);
				CFlowPointcut cflowPC = new CFlowPointcut(methPC);
				defaultPCs.add(cflowPC);
			}
		}
		
		// cast to array
		return castToArray(defaultPCs);
	}

	private static void reconstructLinePointcut(List defaultPCs, IJavaContextualLineBreakpoint bp, Pointcut bpCond) {
		LineExecutionPointcut linePC = new LineExecutionPointcut(bp);
		AfterPointcut afterPC = new AfterPointcut();
		afterPC.setPointcut(linePC);

		// the entire pointcut
		if (isDefaultResidual(bpCond)) {
			defaultPCs.add(afterPC);
		} else {
			AndPointcut andPC = new AndPointcut();
			andPC.addPointcut(afterPC);
			andPC.addPointcut(bpCond.copy());
			defaultPCs.add(andPC);
		}
	}

	private static Pointcut[] castToArray(List defaultPCs) {
		Pointcut[] defaultPCsArray = new Pointcut[defaultPCs.size()];
		int count = 0;
		for (Iterator i = defaultPCs.iterator(); i.hasNext();) {
			Pointcut pc = (Pointcut)i.next();
			defaultPCsArray[count] = pc;
			count++;
		}
		return defaultPCsArray;
	}
}
