package contextualbreakpoints.pointcuts;

import org.eclipse.jdt.internal.debug.core.model.JDIThread;

import com.sun.jdi.IncompatibleThreadStateException;
import com.sun.jdi.Method;
import com.sun.jdi.StackFrame;
import com.sun.jdi.ThreadReference;

import contextualbreakpoints.contextlogic.DebuggerContext;
import contextualbreakpoints.contextlogic.MethodKey;

/**
 * @author Rick
 *
 */
public class CFlowPointcut implements Pointcut {

	
	private Pointcut pointcut;
	
	public CFlowPointcut(Pointcut pc) {
		super();
		setPointcut(pc);
	}
	
	public CFlowPointcut() {
		super();
	}

	/* (non-Javadoc)
	 * @see conditions.Condition#isTrue()
	 */
	public boolean isMatch(JDIThread thread) {
		
		// FUTURE SUPPORT FOR THE GENERAL CASE
		// For now, we only know how to deal with child that is MethodExecution
		// 
		// get relevant info from the child PC and search through stack
		if (!(pointcut instanceof MethodExecutionPointcut)) {
			return false;
		}
		return isMatchMethodExecution(thread);

	}
	
	private boolean isMatchMethodExecution(JDIThread thread) {
		MethodExecutionPointcut methExecPC = (MethodExecutionPointcut)pointcut;
		MethodKey methKey = methExecPC.getMethodKey();
		
		//Method method = DebuggerContext.getInstance().getMethodByName(methKey);
		
		ThreadReference  uThread = thread.getUnderlyingThread();
		try {
			for (int i = 0; i < uThread.frameCount(); i++) {
				StackFrame fr = uThread.frame(i);
//				if (fr.location().method().equals(method)) {
//					return true;
//				}

				Method checkMeth = fr.location().method();
				if (checkMeth.name().equals(methKey.getMethodName()) &&
						checkMeth.signature().equals(methKey.getMethodSignature()) &&
						checkMeth.declaringType().name().equals(methKey.getClassName())) {
					return true;
				}
			}
		} catch (IncompatibleThreadStateException e) {
			e.printStackTrace();
		}
		return false;
	}

	/**
	 * @return Returns the pointcut.
	 */
	public Pointcut getPointcut() {
		return pointcut;
	}

	/**
	 * @param pointcut The pointcut to set.
	 */
	public void setPointcut(Pointcut pointcut) {
		this.pointcut = pointcut;
	}

	public Pointcut copy() {
		CFlowPointcut copy = new CFlowPointcut();
		copy.setPointcut(getPointcut().copy());
		return copy;
	}

}
