package contextualbreakpoints.pointcuts;

import org.eclipse.jdt.internal.debug.core.model.JDIThread;

import com.sun.jdi.Location;
import com.sun.jdi.Method;

import contextualbreakpoints.breakpoints.IJavaContextualLineBreakpoint;
import contextualbreakpoints.contextlogic.DebuggerContext;
import contextualbreakpoints.contextlogic.MethodKey;


public class LineExecutionPointcut implements Pointcut {

	private boolean hasBeenReached;
	private Location[] locations; // locations corresponding to line
	private IJavaContextualLineBreakpoint bp; // origin
	
	/**
	 * @param bp the bp corresponding to the line this pointcut was derived from
	 */
	public LineExecutionPointcut(IJavaContextualLineBreakpoint bp) {
		locations = DebuggerContext.getInstance().getLocations(bp);
		this.bp = bp;
		setBeenReached(bp.hasBeenReached());
		PointcutManager.getInstance().addLineExecutionPointcut(this);
	}
	
	public boolean isMatch(JDIThread thread) {
		// TODO Auto-generated method stub
		return false;
	}
	
	public boolean isMatchLocation(Location loc) {
		for (int i = 0; i < locations.length; i++) {
			Method method1 = locations[i].method();
			MethodKey methKey1 = new MethodKey();
			methKey1.setClassName(method1.declaringType().name());
			methKey1.setMethodName(method1.name());
			methKey1.setMethodSignature(method1.signature());
			
			Method method2 = loc.method();
			MethodKey methKey2 = new MethodKey();
			methKey2.setClassName(method2.declaringType().name());
			methKey2.setMethodName(method2.name());
			methKey2.setMethodSignature(method2.signature());
			
			long line1 = locations[i].codeIndex();
			long line2 = loc.codeIndex();
			if (methKey1.equals(methKey2) && line1 == line2) {
				return true;
			}
		}
		return false;
	}

	public boolean hasBeenReached() {
		return hasBeenReached;
	}
	
	public void setBeenReached(boolean reached) {
		hasBeenReached = reached;
	}

	/**
	 * @return Returns the bp.
	 */
	public IJavaContextualLineBreakpoint getOriginBreakpoint() {
		return bp;
	}

	public Pointcut copy() {
		LineExecutionPointcut copy = new LineExecutionPointcut(getOriginBreakpoint());
		return copy;
	}

}
