package garbage;


import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.model.IBreakpoint;




/**
 * @author Rick
 *
 */
public class DebuggerTriggerEnvironment {

	private DebuggerTriggerContext context;

	private static DebuggerTriggerEnvironment instance;
	
	/**
	 * constructor
	 */
	private DebuggerTriggerEnvironment() {

	}

	/**
	 * @return instance of this class
	 */
	public static DebuggerTriggerEnvironment getInstance() {
		if (instance == null) {
			instance = new DebuggerTriggerEnvironment();
		}
		return instance;
	}

	/**
	 * @return Returns the context.
	 */
	public DebuggerTriggerContext getContext() {
		//System.out.println("returning context");
		
//		if (context != null) {
//			System.out.println("returning context with " + context.getLastTriggerpoint());
//		}
		
		return context;
	}

	/**
	 * @param context The context to set.
	 */
	public void setContext(DebuggerTriggerContext context) {
		this.context = context;
	}
	
	/**
	 * @param breakpoint the last breakpoint reached
	 */
	public void updateContext(IBreakpoint breakpoint) {
		//context = new DebuggerTriggerContext();
		if (context == null) {
			context = new DebuggerTriggerContext();
		}
		// if the breakpoint is a trigger, update context
		if (breakpoint instanceof IJavaTriggerpoint) {
			context.setLastTriggerpoint((IJavaTriggerpoint)breakpoint);
			//System.out.println("updating debugger context with" + breakpoint); //$NON-NLS-1$
		}
	}
	
	/**
	 * resets the trigger context for the debug environment
	 *
	 */
	public void resetContext() {
		context = null;
		
		// and reset all breakpoints to untriggered
		IBreakpoint[] bps = DebugPlugin.getDefault().getBreakpointManager().getBreakpoints();
		for (int i = 0; i < bps.length; i++) {
			if (bps[i] instanceof IJavaTriggeredBreakpoint) {
				IJavaTriggeredBreakpoint tbps = (IJavaTriggeredBreakpoint)bps[i];
				if (tbps.getContext() == null) {
					tbps.setTriggered(true);
				} else {
					tbps.setTriggered(false);
				}
			}
		}
	}
}

